/* iRiver iFP 7XX Emulator
   Copyright (C) 2005 Tomasz Malesinski <tmal@mimuw.edu.pl>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

#include <stdio.h>
#include <stdarg.h>
#include <errno.h>
#include <getopt.h>

char *program_name;

void error_sys(const char *format, ...) {
  va_list ap;
  fprintf(stderr, "%s: ", program_name);
  va_start(ap, format);
  vfprintf(stderr, format, ap);
  va_end(ap);
  fprintf(stderr, "%s\n", strerror(errno));
}

void usage(void) {
  fprintf(stderr, "usage: %s <flash file name> <flash type> [contents]\n"
	  "type is one of data256, data512, data1024, prog1024\n",
	  program_name);
}

#define BUF_SIZE 4096

int main(int argc, char *argv[]) {
  int i, size, fill, done;
  char *fname;
  FILE *fp, *contents_fp;
  unsigned char buf[BUF_SIZE];

  program_name = argv[0];
  if (argc < 3) {
    usage();
    return 1;
  }

  fname = argv[1];
  
  fp = fopen(fname, "wb");
  if (!fp) {
    error_sys("%s: ", fname);
    return 1;
  }

  fill = 0xff;
  if (!strcmp(argv[2], "data256"))
    size = 2112 * (1 << 17);
  if (!strcmp(argv[2], "data512"))
    size = 2112 * (1 << 18);
  if (!strcmp(argv[2], "data1024"))
    size = 2112 * (1 << 19);
  else if (!strcmp(argv[2], "prog1024"))
    size = 1024 * 1024;
  else {
    usage();
    return 1;
  }

  if (argc >= 4) {
    contents_fp = fopen(argv[3], "rb");
    if (!contents_fp) {
      error_sys("%s: ", argv[3]);
      return 1;
    }
  } else
    contents_fp = NULL;

  done = 0;
  while (done < size) {
    int l = (size - done > BUF_SIZE) ? BUF_SIZE : size - done;

    memset(buf, fill, l);
    if (contents_fp) {
      if (fread(buf, 1, l, contents_fp) < l) {
	fclose(contents_fp);
	contents_fp = NULL;
      }
    }

    if (fwrite(buf, l, 1, fp) < 1) {
      error_sys("%s: ", fname);
      return 1;
    }
    done += l;
  }

  if (fclose(fp) == EOF)
    error_sys("%s: ", fname);

  return 0;
}
